@@ -23,9 +23,8 @@ from flask_appbuilder.models.sqla.filters import FilterEqual, FilterStartsWith
 from flask_appbuilder.models.sqla.interface import SQLAInterface
 from flask_appbuilder.views import CompactCRUDMixin, MasterDetailView, ModelView
 from flask_wtf import CSRFProtect
-import jinja2
 
-from .base import FABTestCase
+from .base import BaseMVCTestCase, FABTestCase
 from .const import (
     MODEL1_DATA_SIZE,
     PASSWORD_ADMIN,
@@ -45,7 +44,6 @@ from .sqla.models import (
     TmpEnum,
 )
 
-
 logging.basicConfig(format="%(asctime)s:%(levelname)s:%(name)s:%(message)s")
 logging.getLogger().setLevel(logging.DEBUG)
 
@@ -54,7 +52,6 @@ logging.getLogger().setLevel(logging.DEBUG)
     Constant english display string from framework
 """
 DEFAULT_INDEX_STRING = "Welcome"
-INVALID_LOGIN_STRING = "Invalid login"
 ACCESS_IS_DENIED = "Access is Denied"
 UNIQUE_VALIDATION_STRING = "Already exists"
 NOTNULL_VALIDATION_STRING = "This field is required"
@@ -106,30 +103,8 @@ class MVCBabelTestCase(FABTestCase):
         self.assertEqual(rv.status_code, 302)
 
 
-class BaseMVCTestCase(FABTestCase):
-    def setUp(self):
-        self.app = Flask(__name__)
-        self.app.jinja_env.undefined = jinja2.StrictUndefined
-        self.app.config.from_object("flask_appbuilder.tests.config_api")
-        logging.basicConfig(level=logging.ERROR)
-
-        self.db = SQLA(self.app)
-        self.appbuilder = AppBuilder(self.app, self.db.session)
-
-    @property
-    def registered_endpoints(self) -> Set:
-        return {item.endpoint for item in self.app.url_map.iter_rules()}
-
-    def get_registered_view_endpoints(self, view_name) -> Set:
-        return {
-            item.endpoint
-            for item in self.app.url_map.iter_rules()
-            if item.endpoint.split(".")[0] == view_name
-        }
-
-
 class ListFilterTestCase(BaseMVCTestCase):
-    def test_list_filter_in_valid_object(self):
+    def test_list_filter_invalid_object(self):
         """
         MVC: Test Filter with related object not found
         """
@@ -140,7 +115,49 @@ class ListFilterTestCase(BaseMVCTestCase):
             rv = c.get("/users/list/?_flt_0_roles=-1")
             self.assertEqual(rv.status_code, 200)
 
-    def test_list_filter_unknow_column(self):
+    def test_list_filter_m_m_invalid_object(self):
+        """
+        MVC: Test Filter many to many with related object with invalid type
+        """
+        with self.app.test_client() as c:
+            self.browser_login(c, USERNAME_ADMIN, PASSWORD_ADMIN)
+
+            # Roles doesn't exists
+            rv = c.get("/users/list/?_flt_0_roles=aaaa", follow_redirects=True)
+            self.assertEqual(rv.status_code, 200)
+            if self.db.session.bind.dialect.name != "mysql":
+                data = rv.data.decode("utf-8")
+                self.assertIn("An error occurred", data)
+
+    def test_list_filter_o_m_invalid_object_type(self):
+        """
+        MVC: Test Filter one to many with related object with invalid type
+        """
+        with self.app.test_client() as c:
+            self.browser_login(c, USERNAME_ADMIN, PASSWORD_ADMIN)
+
+            # Roles doesn't exists
+            rv = c.get("/users/list/?_flt_0_created_by=aaaa", follow_redirects=True)
+            self.assertEqual(rv.status_code, 200)
+            if self.db.session.bind.dialect.name != "mysql":
+                data = rv.data.decode("utf-8")
+                self.assertIn("An error occurred", data)
+
+    def test_list_filter_not_o_m_invalid_object_type(self):
+        """
+        MVC: Test Filter one to many with not equal related object with invalid type
+        """
+        with self.app.test_client() as c:
+            self.browser_login(c, USERNAME_ADMIN, PASSWORD_ADMIN)
+
+            # Roles doesn't exists
+            rv = c.get("/users/list/?_flt_1_created_by=aaaa", follow_redirects=True)
+            self.assertEqual(rv.status_code, 200)
+            if self.db.session.bind.dialect.name != "mysql":
+                data = rv.data.decode("utf-8")
+                self.assertIn("An error occurred", data)
+
+    def test_list_filter_unknown_column(self):
         """
         MVC: Test Filter with unknown field
         """
@@ -656,101 +673,6 @@ class MVCTestCase(BaseMVCTestCase):
         data = rv.data.decode("utf-8")
         self.assertIn(DEFAULT_INDEX_STRING, data)
 
-    def test_sec_login(self):
-        """
-            Test Security Login, Logout, invalid login, invalid access
-        """
-        client = self.app.test_client()
-
-        # Try to List and Redirect to Login
-        rv = client.get("/model1view/list/")
-        self.assertEqual(rv.status_code, 302)
-        rv = client.get("/model2view/list/")
-        self.assertEqual(rv.status_code, 302)
-
-        # Login and list with admin
-        self.browser_login(client, USERNAME_ADMIN, PASSWORD_ADMIN)
-        rv = client.get("/model1view/list/")
-        self.assertEqual(rv.status_code, 200)
-        rv = client.get("/model2view/list/")
-        self.assertEqual(rv.status_code, 200)
-
-        # Logout and and try to list
-        self.browser_logout(client)
-        rv = client.get("/model1view/list/")
-        self.assertEqual(rv.status_code, 302)
-        rv = client.get("/model2view/list/")
-        self.assertEqual(rv.status_code, 302)
-
-        # Invalid Login
-        rv = self.browser_login(client, USERNAME_ADMIN, "wrong_password")
-        data = rv.data.decode("utf-8")
-        self.assertIn(INVALID_LOGIN_STRING, data)
-
-    def test_auth_builtin_roles(self):
-        """
-            Test Security builtin roles readonly
-        """
-        client = self.app.test_client()
-        self.browser_login(client, USERNAME_READONLY, PASSWORD_READONLY)
-        # Test authorized GET
-        rv = client.get("/model1view/list/")
-        self.assertEqual(rv.status_code, 200)
-        # Test authorized SHOW
-        rv = client.get("/model1view/show/1")
-        self.assertEqual(rv.status_code, 200)
-        # Test unauthorized EDIT
-        rv = client.get("/model1view/edit/1")
-        self.assertEqual(rv.status_code, 302)
-        # Test unauthorized DELETE
-        rv = client.get("/model1view/delete/1")
-        self.assertEqual(rv.status_code, 302)
-
-    def test_sec_reset_password(self):
-        """
-            Test Security reset password
-        """
-        client = self.app.test_client()
-
-        # Try Reset My password
-        rv = client.get("/users/action/resetmypassword/1", follow_redirects=True)
-        # Werkzeug update to 0.15.X sends this action to wrong redirect
-        # Old test was:
-        # data = rv.data.decode("utf-8")
-        # ok_(ACCESS_IS_DENIED in data)
-        self.assertEqual(rv.status_code, 404)
-
-        # Reset My password
-        rv = self.browser_login(client, USERNAME_ADMIN, PASSWORD_ADMIN)
-        rv = client.get("/users/action/resetmypassword/1", follow_redirects=True)
-        data = rv.data.decode("utf-8")
-        self.assertIn("Reset Password Form", data)
-        rv = client.post(
-            "/resetmypassword/form",
-            data=dict(password="password", conf_password="password"),
-            follow_redirects=True,
-        )
-        self.assertEqual(rv.status_code, 200)
-        self.browser_logout(client)
-        self.browser_login(client, USERNAME_ADMIN, "password")
-        rv = client.post(
-            "/resetmypassword/form",
-            data=dict(password=PASSWORD_ADMIN, conf_password=PASSWORD_ADMIN),
-            follow_redirects=True,
-        )
-        self.assertEqual(rv.status_code, 200)
-
-        # Reset Password Admin
-        rv = client.get("/users/action/resetpasswords/1", follow_redirects=True)
-        data = rv.data.decode("utf-8")
-        self.assertIn("Reset Password Form", data)
-        rv = client.post(
-            "/resetmypassword/form",
-            data=dict(password=PASSWORD_ADMIN, conf_password=PASSWORD_ADMIN),
-            follow_redirects=True,
-        )
-        self.assertEqual(rv.status_code, 200)
-
     def test_generic_interface(self):
         """
             Test Generic Interface for generic-alter datasource
